###You should install R, load necessary packages, and import data sets prior to the workshop, as there will not be internet available at the conference venue. 
##After installing R on your computer, open R and type "getwd()" (without quotes) into the R console.
##R should return the identity of your working directory (e.g., "C:/Users/RLonsinger/Documents")
##From MW.Workshop > DataFiles folder, select the 6 data files and copy these directly to your working directory folder

##Then Run the following code between "##Load Workshop files - Start" and "##Load Workshop files - End" below.
##To run this code, simply copy and past it into the R Console.
##Note, this will attempt to load the 'stringr' package and the necessary files to R. In order for the 'stringr' package to load, you must be online. 
##You may be prompted to select a CRAN mirror. If so, just select a USA CRAN Mirror that is close to you (e.g., KS)
##After running the following code, you can close R, but be sure to select "Yes" to save the workspace image when prompted.

##Load Workshop files - Start
load.conference<-function(){
  stringr.present<-installed.packages()
  packages<-stringr.present[,1]
  if(is.element("stringr", packages)==FALSE){
    install.packages('stringr')
    library('stringr')
  } else library('stringr')  
  WShop.data<-list()   #Identify a list to import workshop data files into
  workshop.data<-c("harvest.data.txt", "harvest.data11.txt", "mt.lion.data08.txt", "mt.lion.data09.txt", "PopEstimates.txt", "ringtail.data.txt")
  na.strings.as<-c(-999, -999, NA, NA, NA, NA)
  wrkshp.data.names<-c("harvest.data", "harvest.data11", "mt.lion.data08", "mt.lion.data09", "PVA.data", "ringtail.data")
  for(i in 1:length(workshop.data)){
    WShop.data[[i]]<-assign(wrkshp.data.names[i],read.table(file=paste(getwd(),workshop.data[i],sep="/"), header=TRUE, sep="\t", na.strings=na.strings.as[i]))
    }
  return(WShop.data) 
}
MW.data<-load.conference()
##Load Workshop files - End


#Benefits of using a supporting R GUI, such as TINN-R or RStudio
          #Save scripts for future use (an important form of metadata for any project/analysis
          #Flexibility in generating lengthy scripts
          #Helps streamline writing code and minimizing errors 
                 #E.g., bracket color coding and autocompletion - In TINN-R this can be adjusted with Options Tab > autocomplete
                 #Color code scripts, and helps minimize errors (e.g., by changing to a designated color if the function is spelled correctly)
                 #Shortcuts to some regularly used functions, e.g., ls()
                 #shift-control-up

x<-1:10     # the ":" indicates a series. In this case, 1 through 10. 
x           #Will call the vector generated above that was stored in "x"

#Help and Search functions
help(fisher.test)
?fisher.test

#Fuzzy help and search functions
help.search("fisher")  #Note that help.search requires quotes!!
??fisher

#The example function can be used to run examples presented in the functions support documentation
example(fisher.test)

#Operators such as <- are generally shortcuts for methods().
#For example, the same could be generated by replacing <- with assign(), and : with seq()
assign("x", seq(1,10,1))
x

#rnorm() details from help - rnorm(n, mean=0, sd=1)
#Indicates that n is a required input, and that mean and sd will use defaults unless otherwise changed
rnorm(100) #Generates 100 random points from a normal distribution with mean = 0 and sd = 1 (the defaults)
rnorm(100, mean=2, sd=1) #Generates 100 random points from a normal distribution with mean = 2 and sd = 1 
rnorm(100, 2, 1) #Argument names omitted, but same as above because all arguments are included and in the correct order
rnorm(sd=1, mean=2, n=100) #Arguments switched, but because they are named, same as above

ls()                       #List all objects in your workspace
ls(pattern="transect")     #Search objects in workspace by using the pattern argument - returns all objects containing the pattern
ls(pattern="^transect")    #Search objects in workspace by using the pattern argument with a grep operator - returns all objects starting the pattern

#Variables store simple values, results of a method, or complex 
x<-6       #x gets 6
y<-4       #y gets 4
x+y
z<-x+y     #substitution occurs at the time of assignment
z

Y<-c(1:3,5:8)*8 #Store results as a vector
Y       

Z<-sqrt(Y)
Z

#Variable types and structures
canids<-c("fox", "wolf", "jackal", "fox")
canids

canids<-factor(canids)   #convert canids vector to factors
canids

foxes<-canids=="fox"     #Store logical vector indication which are foxes
foxes

canids<-as.numeric(canids)      #As a numeric vector
canids

#Referencing Elements
Z[4]         #reference a single element
Z[4:6]       #reference a series of sequential elements
Z[c(4,6)]    #reference multiple nonsequential elements

#Referencing by condition
Z[Z>4 & Z<7]    #Values from the vector between 4 and 7
Z[ceiling(Z)==7]#Values that round up to 7

#Manually load a small amount of data 
Animal.ID<-c("A1","A2","A3","A4","A5")
total.mass<-c(3.3, 2.9, 2.8, 3.0, 3.4)
left.ear<-c(110.2, 105.6, 97.5, 101.5, 107.3)
hind.foot<-c(33.5, 33.9, 32.1, 32.8, 33.4)

body.data<-data.frame(Animal.ID, total.mass, left.ear, hind.foot)
body.data

#Referencing dataframe by location
body.data[1,]          #All of row 1
body.data[,1]          #All of column 1
body.data[2,3]         #Element in Row 1 and Column 3

#Referencing a data frame by $ notation
body.data[,2]
body.data$total.mass

#Referening a data frame by conditions
body.data$total.mass[c(1,4:5)]                                                                  
body.data$total.mass[body.data$total.mass>=3] #reference only those values meeting some criteria

#Alternatives ways to reference the same data
body.data[,2][body.data$total.mass>=3]
body.data[,2][body.data[,2]>=3]

#Pull Harvest data from your imported List
harvest.data<-MW.data[[1]]

#Or, if you did not import all of the data at the beginning you could use someting like below to import the Harvest data only - need to remove # to comment
#getwd()    #Identify your working directory - it may be easiest intially to store your data in your working directory
#data.file<-"C:/Users/RLonsinger/Documents/R_Datasets/harvest.data.txt"   #Identify path to file, include file extension
#harvest.data<-read.table(file=data.file, header=TRUE, sep="\t", na.strings=-999) #Use the read.table function to import data file

head(harvest.data) #Look at the imported data
str(harvest.data)  #check the formatting of the imported data

#May want to code Species and Sex as factors, as these are not actually numeric
harvest.data<-within(harvest.data,Species<-factor(Species))
harvest.data<-within(harvest.data,Sex<-factor(Sex, labels=c("M","F")))   #Also change labels to M and F

str(harvest.data)

#Exporting Data files
data.file<-"C:/Users/RLonsinger/Documents/R_Datasets/harvest.data.modified.txt"   #Identify path to store the file
write.table(harvest.data, file=data.file, quote=FALSE, row.names=FALSE, sep="\t")

#Sourcing in an R Script
source("~/R_Datasets/ConGenR/ConGenR.r") #~ Indicates that the pathname extends from the workind directory, .r is the file extension for many R scripts
load.ConGenR()  #Function that loads the functions associated with ConGenR and additional supporting packages.

#Pull Harvest data from your imported List
harvest.data11<-MW.data[[2]]

#Or, if you did not import all of the data at the beginning you could use someting like this to import the Harvest data from 2011 only
#Importing Harvest data for the second year
#data.file<-"C:/Users/RLonsinger/Documents/R_Datasets/harvest.data11.txt"   #Identify path to file, include file extension
#harvest.data11<-read.table(file=data.file, header=TRUE, sep="\t", na.strings=-999) #Use the read.table function to import data file

names(harvest.data)                        #View names of columns for original data
names(harvest.data11)                      #View names of columns for original data

#Alternativly, you could compare these with:
names(harvest.data)==names(harvest.data11)

#Before you can combine these two datasets, you need to make sure they have the same columns (in the same order)
#Add a column for the year to the orginal data, and enter the year 2010 for each entry
#Store the data with the new column as a new data frame so as to not overwrite the original data.
cbind(harvest.data, Year=rep(2010,length(harvest.data[,1])))   #Binds the year info and prints to screen
harvest.data10<- cbind(harvest.data, Year=rep(2010,length(harvest.data[,1])))   #Store as new symbol

View(harvest.data10) #Look at data
str(harvest.data10)  #check the formatting of the imported data

#Can view the structure of both, but recall that we already changed to format of the 2010 data, so we should do the same for the 2011 data
harvest.data11<-within(harvest.data11,Species<-factor(Species))
harvest.data11<-within(harvest.data11,Sex<-factor(Sex, labels=c("M","F")))   #Also change labels to M and F

#Now combine the 2 datasets by row (rbind)
harvest.data.combined<-rbind(harvest.data10, harvest.data11) #Store as new "symbol" or variable name
harvest.data.combined[c(1:3,48:50),]
View(harvest.data.combined)

#Pull mountain lion data from your imported List
mt.lion.data08<-MW.data[[3]]
mt.lion.data09<-MW.data[[4]]

#Or, if you did not import all of the data at the beginning you could use someting like this to import the lion data only
#Import Mountain Lion data
#data.file08<-"C:/Users/RLonsinger/Documents/R_Datasets/mt.lion.data08.txt"   #Identify path to file, include file extension
#data.file09<-"C:/Users/RLonsinger/Documents/R_Datasets/mt.lion.data09.txt"   #Identify path to file, include file extension
#mt.lion.data08<-read.table(file=data.file08, header=TRUE, sep="\t") #Use the read.table function to import data file, No missing data
#mt.lion.data09<-read.table(file=data.file09, header=TRUE, sep="\t") #Use the read.table function to import data file, No missing data

#Check the structure of the files
#Then, if all looks good, merge based on the common field of the animal's ID. Store as a new object/symbol so as to not overwrite the otherfiles
mt.lion.data0809<-merge(mt.lion.data08,mt.lion.data09)

#How could we drop columns if there are some data that do not want to include?
mt.lion.data0809[,-c(2,4)]	  # - indicates that columns 2 and 4 should be dropped
                              #Would need to store it if we wanted to retain this version of the data frame
                                
#How could we reorder these columns?
mt.lion.data0809[,c(1,2,4,3,5)]	#Providing the column indices in the desired order reorders the columns
                                #Any omitted column indices would be omitted from the resulting data frame
                                #Would need to store it if we wanted to retain this version of the data frame

#Sort combined lion dataset by lion's weight at a time of initial capture
sort(mt.lion.data0809)      #Will not work to sort a data frame - will return an error
sort(mt.lion.data0809$Weight1, decreasing=TRUE)     #Returns a vector of sorted weights, but does not sort the data frame

order(mt.lion.data0809$Weight1)                     #Returns the sort order of the weights in the vectore mt.lion.data0809$Weights1
ord<-order(mt.lion.data0809$Weight1, decreasing=TRUE)    #Return the sort order in decreasing order, stores to the symbol "ord"
mt.lion.data0809<-mt.lion.data0809[ord,]                 #Sort data frame by using the sort order (ord) as the row reference
head(mt.lion.data0809)                                   #View to ensure the data frame is sorted - or use View()

#Select only males from the lion dataset
#Requires you to first be able to easily identify which are males vs females - Need to split the sex out of the ID column
#Lets use this to practice installing and loading packages - the 'stringr' package has a str_sub() function, which will facilitate this
library(stringr)  #Load the stringr package
str_sub(mt.lion.data0809$PUCO.IDs,-1,-1)          #Explore the str_sub() function - returns a CHARACTER vector 
factor(str_sub(mt.lion.data0809$PUCO.IDs,-1,-1))  #Combine with the factor() function to return a FACTOR vector

#Attach a column indicating individual sexes to the data frame
mt.lion.data0809<-cbind(mt.lion.data0809, Sex=factor(str_sub(mt.lion.data0809$PUCO.IDs,-1,-1)))
head(mt.lion.data0809)                    #View data 

#Subset the data frame to return only Males
subset(mt.lion.data0809, mt.lion.data0809$Sex=="M") #Subset data frame - prints to screen
mt.lion.data0809M<-subset(mt.lion.data0809, mt.lion.data0809$Sex=="M") #Save a new symbol indication Males (M)

#Use trees dataset available from data(trees) to explore basic summary statistics
head(trees)
is.na(trees)       #Logical test to see if NA values are present
sum(is.na(trees))  #Sum logical test results (TRUE =1, FALSE =0) to summarize number of missing values

median(trees$Girth); mean(trees$Girth)       #Return both the median and mean girths from the trees dataset
sd(trees$Girth); var(trees$Girth)            #Return both the standard deviation and variance of girths from the trees dataset
min(trees$Girth); max(trees$Girth)           #Return both the minimum and maximum tree girths from the trees dataset
range(trees$Girth)                           #Return both the range (i.e., median and mean) of girths from the trees dataset

#Use the summary() to return a set of summary statistics
summary(trees$Girth)#Return summary for Girths only
summary(trees)      #Return summary for all columns of the trees data set

#Dealing with NAs - NAs to not always respond to methods in the same way that other values do
x<-c(0:10,NA)     #Create a vector with the values 0 to 10 and ending with a NA

x[x!=0]		#Return the vector, excluding those elements that were 0

#This will not work in the same way with NAs though (it will return NA for every element/index)
x[x!=NA]

#Instead, need to apply the is.na()
x[!is.na(x)]	#Remove NAscompare to x[x!=NA]

x[is.na(x)]<-0	#Replace NAs with a value, in this case with 0
x<-na.omit(x)	#drop NAs from vector x  

       
###Afternoon Session

#Pull ringtail data from your imported List
ringtail.data<-MW.data[[6]]

#Or, if you did not import all of the data at the beginning you could use someting like this to import the ringtail data only
#Import ringtail.data
#data.file<-"C:/Users/RLonsinger/Documents/R_Datasets/ringtail.data.txt"
#ringtail.data<-read.table(file=data.file,  header=TRUE, sep="\t", na.strings=NA) 

#Check the imported data structure
head(ringtail.data)
str(ringtail.data)

length(ringtail.data[,1] #How many rows of data are there?
length(ringtail.data$ID)  #Or by $name, see how many rows of data there are.

sapply(ringtail.data[,5:10], mean, na.rm=TRUE)                    #Use an implicit loop to get the mean of each of the numeric variables
tapply(ringtail.data$WEIGHT, ringtail.data$SEX, mean, na.rm=TRUE) #Use an implicit loop to get the mean weight by sex (note: sex must be a factor)
tapply(ringtail.data$WEIGHT, ringtail.data$AGE, mean, na.rm=TRUE) #Use an implicit loop to get the mean weight by age

#Demonstrate order of operations
3 * 3 + 6      #15, Multiplication before addition
3 * (3 + 6)    #27, evaluates within the parentheses first

#Brackets
#General indexing, but also can be useful for subsetting
subset(mt.lion.data0809, mt.lion.data0809$Sex=="M")     #Subset a data frame to return only the males
mt.lion.data0809[mt.lion.data0809$Sex=="M",]            #Could also select the same subset by using a conditional statement within the referencing brackets
mt.lion.data0809[mt.lion.data0809$Sex=="M" & mt.lion.data0809$Weight2>125,]    #Return males >125 lbs during year 2

#We could combine the () and [] notions 
mt.lion.data0809[mt.lion.data0809$Sex=="M" & (mt.lion.data0809$Weight2>125 | mt.lion.data0809$Weight2<120),] #Return lions >125 and <118 lbs that are M - Evaluates the weight first

#Conditional Statements
if(trees$Height[1]>trees$Girth[1]) "Correct"                               # Does not require the false response, if false, does nothing [1] "Correct"
if(trees$Height[1]>trees$Girth[1]) "Correct" else "Potential Issues"       #[1] "Correct"
if(trees$Height[1]<trees$Girth[1]) "Correct" else "Potential Issues"       #[1] "Potential Issues"

a <- c(6:-4)
sqrt(a)                           #gives warning - returns NaN for negative values
ifelse(a >= 0, a, NA)             #Only return value if it is positive 
sqrt(ifelse(a >= 0, a, NA))       #Pass only positive values (logical >=0) to sqrt function

#repeat loops
i <- 10
repeat {
       if(i > 100) break else {print(i); i <- i + 10}  #use if else statement with break keyword to avoid an infinate loop
       }

#While loops - Do the same as above, but now with a while loop. 
i <- 10
while (i <=100) {
      print(i); i <- i + 10
	    }

#For loops - do the same as above, but now with a for loop
b <- seq(10, 100, 10)
for (i in b) print(i)
#Or
for (i in seq(10, 100, 10)) print(i)

#For loops - do the same as above, but now using 1:length() to move through all elements of the vector/list
b <- seq(10, 100, 10)
for (i in 1:length(b)) print(b[i])

#Functions often used to control looping functions and/or store resulting output
length()                 #Return the length of a vector
length(1:10)             #Better be 10!
length(ringtail.data)    #If given a data frame, length will return the number of oclumns - Often though, we want the number of rows
length(ringtail.data[,1])#Return length of column 1 (i.e., the number of rows in the dataframe)

sum( )                   #Return the total cumulative sum of arguments
sum(1:10)                #Return the result of 1+2+...+10 =  55         

c( )                     #Concatenate or string together values, in the order provided
c(1:5, 10)               #Returns [1]  1  2  3  4  5 10
c(10, 1:5)               #Returns [1]  10 1  2  3  4  5 

append( )                #Appends values at a specified location - defaults to the end of
append(1:5, 10)          #Returns [1]  1  2  3  4  5 10
append(1:5, 10, 0)       #Returns [1]  10 1  2  3  4  5 
append(1:5, 10, 1)       #Returns [1]  1 10  2  3  4  5 - Offers the flexibility to append in the middle of the initial vector

#With the mt.lion.data0809,
#1. Calculate the change in body weight for each animal between 2008 and 2009 captures
Delta.weight<-NULL	#Create an empty vector

for(i in 1:length(mt.lion.data0809$PUCO.IDs)){
	Delta.weight<-append(Delta.weight, mt.lion.data0809$Weight2[i]- mt.lion.data0809$Weight1[i])
	}

#2. Store this new metric as a column in the data frame
mt.lion.data0809<-cbind(mt.lion.data0809, Delta.weight)

#3. Select only those animals that have lost weight
mt.lion.data0809[mt.lion.data0809$Delta.weight<0,]#Could also use subset()

#####USER DEFINED FUNCTIONS                      
SE = sd(x)/sqrt(length(x))
#For some vector x, standard error is:
sd(x)/sqrt(length(x)) #Could use this to calculate a Standard Error
sd(x, na.rm=TRUE)/sqrt(length(x)-sum(is.na(x))) #But you may need to get more complicated if your vector contains missing values

#Rather than do this every time, you could just build a simple function that calculates SE, and accounts for missing values
SE <- function (x){
	sd(x, na.rm=TRUE)/sqrt(length(x)-sum(is.na(x)))
	}

SE.list <- function (x){
	SE<-sd(x, na.rm=TRUE)/sqrt(length(x)-sum(is.na(x)))
  return(list(Mean = mean(x, na.rm=TRUE), SE = SE, NA.removed = sum(is.na(x))))
	}
  

#PVA Simulation Example

#Pull Population estimate data from your imported List
PVA.data<-MW.data[[5]]

#Or, if you did not import all of the data at the beginning you could use someting like this to import the populations estimate data only
#Import population estimate data
#data.file<-"C:/Users/RLonsinger/Documents/R_Datasets/PopEstimates.txt"
#PVA.data<-read.table(file=data.file,  header=TRUE, sep="\t", na.strings=NA) 

#Calculate the population growth rate (lambda) for each year, where Lambda = Nt+1/Nt 
lambda<-NULL
for(i in 1:length(PVA.data$N)-1){ #Need to consider that the lambda can only be calculated if Nt+1 has a value
      lambda<-append(lambda, PVA.data$N[i+1]/PVA.data$N[i])
      }
mean.lambda<-mean(lambda)#Calculate the mean of lambda
sd.lambda<-sd(lambda)    #Calculate the SD of lambda
                                           
#Simulate population viability for 100 years - Assume Lamda~Normal(mean, SD) 
Nhat<-PVA.data$N[length(PVA.data$N)]#Initialize/Specify the starting population size as the last estimated abundance

#Randomly draw a population growth rate from the distribution of observed rates use this to predict the next Nhat
Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda)

#To randomly select lambda and estimate Nhat for each year, need to build this into a loop, store the result of each loop, and calculate each Nhat from the previous year's Nhat
#Initialize starting values, then build loop to generate single run of a count based PVA 
Nhat<-PVA.data$N[length(PVA.data$N)]#Initialize/Specify the starting population size as the last estimated abundance
years<-100
for(year.i in 1:years){
  Nhat<-append(Nhat, Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda))          
  }
#Note: This simple loop assumes the population is not constrained by 0 and can therefore recover from 0 or negative values - probably not a great assumptions
#sum(Nhat<0) #This code can be used to see how many values were <0

#Alternatively, we could use some conditional statements to ensure that once a population gets to 0, it stays there
Nhat<-PVA.data$N[length(PVA.data$N)]#Initialize/Specify the starting population size as the last estimated abundance
years<-100
for(year.i in 1:years){
  if(Nhat[length(Nhat)]>0){ 
    Nhat<-append(Nhat, Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda))
    } else {
    Nhat<-append(Nhat, 0)
    }          
  }

#Conduct 1000 iterations, storing the results of each
sims<-1000 #Initialize the number of simulations 
years<-100  #Initialize the number of years
#Note, both of these can be outside both of the looping constructs, because their values never change within the simulation

PVA.results<-data.frame(row.names=c(1:(years+1)))     #Create EMPTY data frame with the same number of rows as the Nhat output that will be stored
for(sim.i in 1:sims){
  Nhat<-PVA.data$N[length(PVA.data$N)]#Now need to reinitilize/specify for each sim.i, so inside outer loop
  #Determine the stochastic outcome of the population over 100 years, as we just did above in a for loop
  for(year.i in 1:years){
    if(Nhat[length(Nhat)]>0){ 
      Nhat<-append(Nhat, Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda))
      } else {
      Nhat<-append(Nhat, 0)
      }          
    }
  #Store the results of the nested loop
  PVA.results<-cbind(PVA.results, Nhat)  #Each results set is stored as a column
  }
#Summarize the result as Pr(Extinction)
#Note, in the output (PVA.results), each column is a result, and the final Row can be used to quickly summarize the Prob(Extinction) over the next 100 years
sum(PVA.results[nrow(PVA.results),]<=0)/sims #Proportion of simulations in which the population went to extinction


##Loop over increased values for the sd.lambda parameter
sims<-1000 #Initialize the number of simulations 
years<-100  #Initialize the number of reps 
sd.lambda<-c(sd.lambda, (sd.lambda+0.1), (sd.lambda+0.2))     #Identify the levels of sd.lambda to consider

PVA.results.bySD<-list()   #Identify a list in which you can store all of your results
Prob.Ext.bySD<-NULL        #Identify an empty vector to store the Probability of Extinction resulting from each of the 3 SD levels

for(lambda.i in 1:length(sd.lambda)){
  #Conduct 1000 iterations, storing the results of each  
  #################################################################Start the loop developed above for single PVA - now a nested loop
  PVA.results<-data.frame(row.names=c(1:(years+1)))
  for(sim.i in 1:sims){
    Nhat<-PVA.data$N[length(PVA.data$N)]
    for(year.i in 1:years){
      if(Nhat[length(Nhat)]>0){ 
        Nhat<-append(Nhat, Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda[lambda.i]))  
        #only change is here-added lambda.i indexing to reference the appropriate sd.lambda from outer loop
        } else {
        Nhat<-append(Nhat, 0)
        }          
      }
    #Store the results of the most nested loop
    PVA.results<-cbind(PVA.results, Nhat)  #Each results set is stored as a column
    }
  ##################################################################End loop developed above for single PVA
  PVA.results.bySD[[lambda.i]]<-PVA.results  #Store the result of the nested loop
  #Summarize the result as Pr(Extinction)
  Prob.Ext.bySD[lambda.i]<-sum(PVA.results[nrow(PVA.results),]<=0)/sims #Proportion of sims in which the N went to extinction
  }
Prob.Ext.bySD<-cbind(sd.lambda, Prob.Ext=Prob.Ext.bySD)         #convert to data frame with lambda SD levels
PVA.results.bySD[[length(PVA.results.bySD)+1]]<- Prob.Ext.bySD  #Attach Prob Extinction results to PVA simulation results list (to keep all of the results together)


################To make this easier to use in the future:
################Convert PVA code to a User defined function
#PVA Simulation Example
#Import population estimate data
PVA.data<-MW.data[[5]]
#Note: PVA.data required and should have 1 column named 'N', which contains the population estimates over time
#Note: Any other columns will be ignored
#Note: years = number of time steps for which viability is assessed. 
#Note: Sims = number of PVA simulations used to estimate Prob(Extinction)
#Note: Added and ext.thresh argument. Allows the user to change the criteria from 0 to some other threshold 
CB.PVA<-function(PVA.data, mean.lambda=NULL, sd.lambda=NULL, initial.Nhat=NULL, ext.thresh=0,  years=50, sims=20){ 
  lambda<-NULL              #Indentify empty vector to store lambda values
  PVA.results.bySD<-list()  #Identify a list in which you can store all of your results
  Prob.Ext.bySD<-NULL       #Identify an empty vector to store the Probability of Extinction resulting from each of the n possible SD levels

  for(i in 1:length(PVA.data$N)-1){  #Need to consider that the lambda can only be calculated if Nt+1 has a value
    lambda<-append(lambda, PVA.data$N[i+1]/PVA.data$N[i])
    }
  #Added some control with conditional statements - Depends on arguments passed to function
  if(is.null(mean.lambda)){  
    mean.lambda<-mean(lambda)#Calculate the mean of lambda  
    } else mean.lambda<-mean.lambda
  if(is.null(sd.lambda)){
    sd.lambda<-sd(lambda)    #Calculate the SD of lambda  
    } else sd.lambda<-sd.lambda
  if(is.null(initial.Nhat)) Nhat<-PVA.data$N[length(PVA.data$N)] else Nhat<-initial.Nhat
  return.Nhat<-Nhat         #Add a place to store the initial.Nhat applied to the subsequent loops

  for(lambda.i in 1:length(sd.lambda)){
    PVA.results<-data.frame(row.names=c(1:(years+1)))    
    for(sim.i in 1:sims){
    if(is.null(initial.Nhat)) Nhat<-PVA.data$N[length(PVA.data$N)] else Nhat<-initial.Nhat
      for(year.i in 1:years){
        if(Nhat[length(Nhat)]>0){ 
          Nhat<-append(Nhat, Nhat[length(Nhat)]*rnorm(1, mean=mean.lambda, sd=sd.lambda[lambda.i]))
          } else {
          Nhat<-append(Nhat, 0)
          }          
        }
      PVA.results<-cbind(PVA.results, Nhat)  
      }
  PVA.results.bySD[[lambda.i]]<-PVA.results
  Prob.Ext.bySD[lambda.i]<-sum(PVA.results[nrow(PVA.results),]<=ext.thresh)/sims
  }
  Prob.Ext.bySD<-cbind(Nhat=rep(return.Nhat, length(sd.lambda)),
                       mean.lambda=rep(round(mean.lambda,2), length(sd.lambda)),
                       sd.lambda,
                       Threshold=rep(ext.thresh, length(sd.lambda)), 
                       Prob.Ext=Prob.Ext.bySD) 
  #Now including mean.lambda in output
  PVA.results.bySD[[length(PVA.results.bySD)+1]]<- Prob.Ext.bySD          
  return(PVA.results.bySD = PVA.results.bySD)
  } #End function

##Try it out with a sequence of SD values and for a threshold of 10 (leaving all other values as defaults)
##So years=50, sims=20
temp.results<-CB.PVA(PVA.data, sd.lambda=c(0.1, 0.2, 0.3), ext.thresh=10)
  
##Now if we want to test the influence of initial starting size, we can simply build a shorter script, which calls this function!!
initial.Nhat<-c(20,30,40)
Results<-list()  #Create structure for holding data

for(Nhat.i in 1:length(initial.Nhat)){ 
  #Now call user defined function, setting initial.Nhat to sequential (Nhat.i) values
  temp.results<-CB.PVA(PVA.data, initial.Nhat=initial.Nhat[Nhat.i], sd.lambda=c(0.1,0.2,0.3), ext.thresh=10) 
  Results[[length(Results)+1]]<-temp.results  
  }
#Note: These results are not being returned in the neatest fashion, but this is not always the greatest concern (but could be if you expect others to use your ocde!)
                                                                                                                 
#Lets pull the data and summarize 
#View the summary results for each Nhat value 
Results[[1]][[4]]         #Reference list element 4 of list element 1
Results[[2]][[4]]         #these are matrices, not data frames (use class() to verify)
Results[[3]][[4]]

#Pull these together into a data frame 
temp<-data.frame(rbind(Results[[1]][[4]],Results[[2]][[4]],Results[[3]][[4]]))


##Graphing and Plotting

#Recall above that we considered 3 levels SD in lambda and 3 levels of initial Nhat at 40 (these results were stored as Results

plot(Results[[1]][[1]][,1])                #Quick exploratory scatterplot of one simulated population trajectory
plot(Results[[1]][[1]][,1], type="l")      #Probably want to change from points to a line, with argument type="l"

for(rep.i in 2:length(Results[[1]][[1]])){ #Loop over all of the remaining 19 replicate population trajectories
  lines(Results[[1]][[1]][,rep.i], type="l")         #Add a line to the plot for this replicate
  }

#Notice:
#It can be difficult to follow partilar trajectories due to them all being the same color - We can fix this if we want!
#Some go off the plot - The extent of the first plot is maintained when new data is added, even if that data falls outside the plot boundaries.
plot(Results[[1]][[1]][,1], type="l", ylim=c(0, max(Results[[1]][[1]])))    #Set new y axis limits
for(rep.i in 2:length(Results[[1]][[1]])){    
  lines(Results[[1]][[1]][,rep.i], type="l", col=rep.i)  #Add the col argument, set equal to rep.i to loop through the colors
  }

#Can clean up the labeling
plot(Results[[1]][[1]][,1], type="l", ylim=c(0, max(Results[[1]][[1]])), xlab="Years", ylab="N-hat")    #label axes
for(rep.i in 2:length(Results[[1]][[1]])){    
  lines(Results[[1]][[1]][,rep.i], type="l", col=rep.i)         
  }

#These initial PVA plots are for a specific initial Nhat value and standard deviation of lambda
#But our results file now contains 3 PVA analysis for each of 3 SD levels

#How could we plot all of these together?
par(mfrow=c(3,3))    #Create plots in 3 rows, and 1 columns, and fill row-wise, i.e., row 1 first, then row 2, then row 3

for(plot.i in 1:length(Results)){
  for(sd.i in 1:(length(Results[[plot.i]])-1)){
    plot(Results[[plot.i]][[sd.i]][,1], type="l", ylim=c(0, 750), xlab=NA, ylab=NA)
        legend("topleft", c(paste("SD = ", Results[[plot.i]][[4]][sd.i,3], sep=""),
                      paste("Prob Ext = ", Results[[plot.i]][[4]][sd.i,5], sep=""),
                      paste("Nhat = ", Results[[plot.i]][[4]][sd.i,1], sep="")))
      for(rep.i in 2:length(Results[[plot.i]][[sd.i]])){
      lines(Results[[plot.i]][[sd.i]][,rep.i], type="l", col=rep.i)
      }
    }
  }
  
##Code used to generate the arguments plot
#####Graphics and Plotting

#Create plot of plotting arguments
par(las=1, mar=c(4,4,4,6))
plot(c(1:25),rep(14,25), pch=1:25, cex=1.5, ylim=c(0,14), xlab=NA, ylab=NA, axes=FALSE)
points(c(1:25),rep(13,25), pch=1, cex=seq(0.1,2.5,0.1))
points(c(1:25),rep(12,25), pch=16, cex=1.5, col=1:25)
points(seq(0,25, 5),rep(11,6), pch=1, cex=1.5, type="p")
points(seq(0,25, 5),rep(10,6), pch=1, cex=1.5, type="c")
points(seq(0,25, 5),rep(9,6), pch=1, cex=1.5, type="b")
points(seq(0,25, 5),rep(8,6), pch=1, cex=1.5, type="o")
points(seq(0,25, 5),rep(7,6), pch=1, cex=1.5, type="l")
abline(6,0, lwd=3, lty=1)
abline(5,0, lwd=3, lty=2)
abline(4,0, lwd=3, lty=3)
abline(3,0, lwd=3, lty=4)
abline(2,0, lwd=3, lty=5)
abline(1,0, lwd=3, lty=6)

lines(x=c(1,2), y=c(0,0), lwd=1)
lines(x=c(3,4), y=c(0,0), lwd=2)
lines(x=c(5,6), y=c(0,0), lwd=3)
lines(x=c(7,8), y=c(0,0), lwd=4)
lines(x=c(9,10), y=c(0,0), lwd=5)
lines(x=c(11,12), y=c(0,0), lwd=6)
lines(x=c(13,14), y=c(0,0), lwd=7)
lines(x=c(15,16), y=c(0,0), lwd=8)
lines(x=c(17,18), y=c(0,0), lwd=9)
lines(x=c(19,20), y=c(0,0), lwd=10)
lines(x=c(21,22), y=c(0,0), lwd=11)
lines(x=c(23,24), y=c(0,0), lwd=12)

box()
axis(1, at=seq(1.5,23.5,2), labels=1:12, tick=TRUE)
axis(2, at=7:11, labels=c('"l"', '"o"', '"b"', '"c"', '"p"'), tick=TRUE)
axis(3, at=seq(1,25,3), labels=TRUE)
axis(4, at=1:6, labels=c(6:1), tick=TRUE)
axis(4, at=12:14, labels=c("col: color","cex: size","pch: type"), tick=TRUE)

mtext("lwd: line width", side=1, line=2.5, las=1, cex=2)
mtext("type: points()", side=2, line=2.5, las=3, cex=2, adj=.7)
mtext("Points", side=3, line=2, las=1, cex=2)
mtext("lty: line type", side=4, line=2.5, las=0, cex=2, adj=.15)
mtext("Arguments", side=4, line=1, las=1, cex=1, padj=-20)